@@ -1,12 +1,12 @@
-/**********************************************************
+/*********************************************************
  *
- * Snare for AIX version 1.4
+ * Snare for AIX version 1.5.0
  *
  * Author: InterSect Alliance Pty Ltd
  *
- * Copyright 2001-2005 InterSect Alliance Pty Ltd
+ * Copyright 2001-2007 InterSect Alliance Pty Ltd
  *
- * Last Modified: 27/10/2005
+ * Last Modified: 5/12/2007
  *
  * Available under the terms of the GNU Public Licence.
  * - See www.gnu.org
@@ -27,6 +27,10 @@
  *       7/10/2004  Test version - based on Snare for AIX
  *       27/10/2005 Version 1.1 released 
  *       2/8/2006   Latest updates to fix syslog truncation
+ *       17/4/2007  Remove debug flag and update syslog message format
+ *       23/11/2007 Updated code base including Latest Events
+ *       	    and access to /etc/sudoers
+ *       5/12/2007  Multiple bug fixes in objective matching code
  *
  **********************************************************
  *
@@ -57,6 +61,7 @@
 #include <strings.h>
 #include <values.h>
 #include <time.h>
+#include <pthread.h>
 
 #include <regex.h>
 
@@ -70,6 +75,7 @@
 
 ////// DEBUG
 #include <stdarg.h>
+//#define DEBUG 1
 void DebugMsg(const char *pszFormat, ...);
 ////// END DEBUG
 
@@ -79,6 +85,7 @@ static int AuditDestination = AUDIT_TO_STDOUT;	// Send to STDOUT unless otherwis
 static int SyslogDestination = 13;			// user.notice
 
 char hostid[MAX_HOSTID] = "";
+int restart_allow = 0;
 int remote_allow = 0;
 int remote_webport = 6161;
 char remote_restrictip[16] = "";
@@ -87,22 +94,69 @@ char remote_password[256] = "";
 FILE *AuditFile = (FILE *) NULL;
 //int AuditSocket = 0;
 //struct sockaddr_in AuditSocketName;
+pthread_mutex_t list_mutex = PTHREAD_MUTEX_INITIALIZER; 
+pthread_mutex_t data_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t event_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_cond_t  list_data_available = PTHREAD_COND_INITIALIZER;
 
 int MCCount=0;
 MsgCache *MCHead=NULL;
 MsgCache *MCTail=NULL;
 MsgCache *MCCurrent=NULL;
 
+int EvtCount=0;
+WebCache *EvtHead=NULL;
+WebCache *EvtTail=NULL;
+WebCache *EvtCurrent=NULL;
+
 HostNode *hhead=NULL;
 HostNode *htail=NULL;
 HostNode *hcurrent=NULL;
 
+// Default to UDP delivery
 int SocketType = SOCKETTYPE_UDP;
 
 int HNodeCounter=0;
 
 /* Define an error variable so we can communicate with auditsvc */
 extern int errno;
+int caught_pipe;                // variable that lets us know when a SIGPIPE has been received
+int caught_usr1;                // variable that lets us know when a SIGUSR1 has been received
+int restart_agent=0;            // variable that lets us know whether to restart the agent or not
+
+void pipe_signal(sig)
+{
+	caught_pipe++;
+#ifdef DEBUG
+	DebugMsg("Caught pipe");
+#endif //DEBUG
+	return;
+}
+
+void kill_signal(sig)
+{
+	caught_kill++;
+#ifdef DEBUG
+	DebugMsg("Caught kill");
+#endif //DEBUG
+	if (restart_agent && restart_allow) {
+#ifdef DEBUG
+	DebugMsg("Restarting agent in 5 seconds");
+#endif //DEBUG
+		sleep(5);
+		system("/etc/init.d/snare restart");
+		restart_agent = 0;
+	}
+	return;
+}
+
+void usr1_signal(sig) {
+	caught_usr1++;
+#ifdef DEBUG
+	DebugMsg("Caught usr1");
+#endif //DEBUG
+	return;
+}
 
 int main(int argc, char *argv[])
 {
@@ -114,6 +168,9 @@ int main(int argc, char *argv[])
 	// struct sockaddr_in socketname;
 	// struct in_addr ia;
 
+	// Signal related variables
+	sigset_t signalset;
+
 	/* Audit related variables */
 	char username[MAX_USERNAME];
 	char searchterm[MAX_PATH];
@@ -121,12 +178,12 @@ int main(int argc, char *argv[])
 	char options[MAX_OPTIONS];
 	char *startrecord;
 	int returncode;
-	int webpid = 0;
+	pthread_t thread; 
 	
 	// NODE that matches the objective search term.
 	Node *nodematch;
 
-	DebugMsg("Program starting.\n");
+	DebugMsg("Program starting.");
 	/* The only person who can run auditsvc is root, so lets
 	   make sure that we discourage other users */
 	if (getuid() != 0) {
@@ -145,25 +202,26 @@ int main(int argc, char *argv[])
 		}
 	}
 
+
 	if (remote_allow) {
-		webpid = fork();
-		if (webpid == 0) {
-			// Child process - Run our web server
-			DebugMsg("Starting web server.\n");
-			while (InitWebServer
-			       (remote_webport, remote_restrictip,
-				remote_password) == -1) {
-				// Problem binding. Sleep for a moment.
-				sleep(5);
-			}
-			while ((returncode = StartThread()) == 1) {
-				// Problem binding. Keep trying.
-				sleep(5);
-			}
-			exit(0);
+		DebugMsg("Starting web server.");
+		if (!pthread_create(&thread, NULL, (void *)start_web, NULL)) {
+			//DebugMsg("Web thread created successfully: %d",thread);
+		} else {
+			DebugMsg("ERROR: Web thread failed");
 		}
 	}
-	DebugMsg("entering loop.\n");
+
+	// Configure / Reinstate signals
+	// Trap signals relating to PIPE failures,
+	//  and Child process termination
+	// Reset this each time through the loop, just in case
+	// a signal handler resets our values
+
+	if (!setsignals(&signalset)) {
+		DebugMsg("Cannot set important signals - exiting\n");
+		return(1);
+	}
 
 	while (fgets(logbuffer, sizeof(logbuffer), stdin)) {
 		startrecord = logbuffer;
@@ -172,7 +230,7 @@ int main(int argc, char *argv[])
 		eventname[0] = '\0';
 		options[0] = '\0';
 		returncode = 0;
-		//if (strcmp(searchterm,"/home/u0000190/SNARE-OUT")) DebugMsg("values: == %s == %s == %s == %s == %d\n", username, searchterm, eventname, options, returncode);
+		//if (strcmp(searchterm,"/tmp/SNARE-OUT")) DebugMsg("values: == %s == %s == %s == %s == %d\n", username, searchterm, eventname, options, returncode);
 
 		/* This is where we implement the SNARE event checker process */
 		returncode =
@@ -182,7 +240,7 @@ int main(int argc, char *argv[])
 				  eventname,
 				  options);
 
-		//if (strcmp(searchterm,"/home/u0000190/SNARE-OUT")) DebugMsg("values: == %s == %s == %s == %s == %d\n", username, searchterm, eventname, options, returncode);
+		if (strcmp(searchterm,"/tmp/SNARE-OUT")) DebugMsg("values: == %s == %s == %s == %s == %d", username, searchterm, eventname, options, returncode);
 		while ((nodematch =
 			CheckObjective(username,
 				       searchterm,
@@ -196,24 +254,99 @@ int main(int argc, char *argv[])
 			     nodematch->
 			     criticality) != 0) {
 				// Report an error, but continue
-				DebugMsg("error sending log entry to destination\n");
+				DebugMsg("error sending log entry to destination");
+			} else {
+				//Pop the event in the WebCache
+				EvtCurrent = (WebCache *)malloc(sizeof(WebCache));
+				if (EvtCurrent) {
+					//DebugMsg("******Adding message to event cache [%d].", EvtCount);
+					time_t currenttime;
+					struct tm *newtime;
+					char *psterm;
+
+					time(&currenttime);
+					newtime = localtime(&currenttime);
+					syslogdate(EvtCurrent->CurrentDate, newtime);
+					strncpy(EvtCurrent->hostid,hostid,sizeof(hostid));
+					EvtCurrent->criticality = nodematch->criticality;
+					EvtCurrent->returncode = returncode;
+					strncpy(EvtCurrent->username, username, sizeof(username));
+					strncpy(EvtCurrent->searchterm, searchterm, sizeof(searchterm));
+					psterm = EvtCurrent->searchterm;
+					while (*psterm) {
+						if (*psterm == '%' || *psterm == '<' || *psterm == '>' || *psterm == '&') {
+							*psterm = '_';
+						}
+						psterm++;
+					}
+					strncpy(EvtCurrent->eventname, eventname, sizeof(eventname));
+					strncpy(EvtCurrent->options, options, sizeof(options));
+					EvtCurrent->seenflag=0;
+					EvtCurrent->next = NULL;
+					EvtCurrent->prev = NULL;
+					pthread_mutex_lock(&event_mutex);
+					if (EvtCount >= WEB_CACHE_SIZE) {
+						//Lock Mutex and drop the oldest record
+						WebCache *temp;
+						temp = EvtTail;
+						EvtTail = EvtTail->prev;
+						EvtTail->next = NULL;
+						free(temp);
+						EvtCount--;
+					}
+					if (EvtHead) {
+						EvtHead->prev = EvtCurrent;
+						EvtCurrent->next = EvtHead;
+					}
+					EvtHead = EvtCurrent;
+					if (!EvtTail) EvtTail = EvtCurrent;
+					EvtCount++;
+					pthread_mutex_unlock(&event_mutex);
+				} else {
+					DebugMsg("Unable to allocate latest event cache");
+				}
 			}
 		}
 	}
 	DebugMsg("All done, cleaning up\n");
 
-	if (webpid) {
-		kill(webpid, SIGTERM);
-		waitpid(webpid, NULL, 0);
-	}
-
 	DestroyList();
 
 	/* All done */
 	return 0;
 }
 
+int setsignals(sigset_t *signalset)
+{
+	sigfillset(signalset);
+	sigdelset(signalset,SIGTERM);
+	sigdelset(signalset,SIGALRM);
+	sigdelset(signalset,SIGINT);
+	sigdelset(signalset,SIGPIPE);
+	sigdelset(signalset,SIGUSR1);
+	sigdelset(signalset,SIGCHLD);
+	sigprocmask(SIG_BLOCK,signalset, (void *) NULL);
 
+	if (signal(SIGUSR1,usr1_signal) == SIG_ERR)
+	{
+		DebugMsg("Cannot set signal SIGUSR1\n");
+		return(0);
+	}
+
+	if (signal(SIGPIPE,pipe_signal) == SIG_ERR)
+	{
+		DebugMsg("Cannot set signal SIGPIPE\n");
+		return(0);
+	}
+
+	if (signal(SIGCHLD,pipe_signal) == SIG_ERR)
+	{
+		DebugMsg("Cannot set signal SIGCHLD\n");
+		return(0);
+	}
+
+	return(1);
+}
 // Shortread: don't re-read all the objectives - just the general configuration items.
 int read_config_file(int shortread)
 {
@@ -230,9 +363,9 @@ int read_config_file(int shortread)
 	strncpy(remote_password, "", sizeof(remote_password));
 
 	// Clear the audit destination
-	AuditDestination = 0;
+	AuditDestination=0;
 
-	configfile = fopen(CONFIG_FILENAME, "r");
+	configfile = fopen(CONFIG_FILENAME,"r");
 
 	if (configfile == (FILE *)NULL) {
 		fprintf(stderr,"Cannot open audit configuration file");
@@ -272,91 +405,47 @@ int read_config_file(int shortread)
 						continue;
 					}
 
-					if ((criticality =
-					     splitobjective(inputbuffer,
-							    event, user,
-							    path,
-							    &excludeflag,
-							    &excludematchflag,
-							    &returncode))
-					    > -1) {
+					if ((criticality = splitobjective(inputbuffer, event, user, path, &excludeflag, &excludematchflag, &returncode)) > -1) {
 						// add the objective to the linked list.
 						trimallwhitespace(event);
 
-						// HERE: Split event into comma separated list of events                                
+						// HERE: Split event into comma separated list of events
 						eventpointer = event;
 						eventpointer2 = event;
 
 						// While there are no more commas
 						while (eventpointer2) {
 							eventpointer2 = strstr(eventpointer, ",");
-							if (eventpointer2
-							    ==
-							    (char *) NULL)
-							{
+							if (eventpointer2 == (char *)NULL) {
 								// No commas left. Just copy to the end of the line.
-								strncpy
-								    (event2,
-								     eventpointer,
-								     MAX_AUDIT_CONFIG_LINE);
+								strncpy(event2, eventpointer, MAX_AUDIT_CONFIG_LINE);
 							} else {
-								strncpy
-								    (event2,
-								     eventpointer,
-								     (eventpointer2
-								      -
-								      eventpointer));
+								strncpy(event2, eventpointer, (eventpointer2 - eventpointer));
 								// Make sure we have a null on the end of the line.
-								event2
-								    [eventpointer2
-								     -
-								     eventpointer]
-								    = '\0';
+								event2[eventpointer2 - eventpointer] = '\0';
 							}
 							if (eventpointer2) {
 								// Skip the comma
-								eventpointer
-								    =
-								    eventpointer2
-								    + 1;
+								eventpointer = eventpointer2 + 1;
 							}
 							// Clear out any options strings
-							strcpy(options,
-							       "");
+							strcpy(options, "");
 
-							if (regmatch
-							    (event2,
-							     "\\(.*\\)$"))
-							{
+							if (regmatch(event2, "\\(.*\\)$")) {
 								char *position;
 
 								// Bonus, we have something. Grab it, add to options.
 								// First, trim off the last bracket.
-								event2
-								    [strlen
-								     (event2)
-								     - 1] =
-								    '\0';
-								position =
-								    strstr
-								    (event2,
-								     "(");
+								event2[strlen(event2) - 1] = '\0';
+								position = strstr(event2, "(");
 
-								strncpy
-								    (options,
-								     position
-								     + 1,
-								     MAX_OPTIONS);
+								strncpy(options, position + 1, MAX_OPTIONS);
 								// truncate the event string at the bracket.
-								*position =
-								    '\0';
+								*position = '\0';
 							}
  
 							//DebugMsg("About to AddToList...\n");
-							if (!strcmp
-							    (event,
-							     "*"))
-							{
+							if (!strcmp(event, "*")) {
 								AddToList
 								    ("AUDIT_ALL",
 								     user,
@@ -379,80 +468,45 @@ int read_config_file(int shortread)
 							}
 						}
 					} else {
-						DebugMsg(
-							"WARNING: Cannot process objective - please ensure configuration line\n\tcontains valid criticality, event, criticality, return, user and match elements\n\t[%s]\n",
-							inputbuffer);
+						DebugMsg("WARNING: Cannot process objective - please ensure configuration line\n\tcontains valid criticality, event, criticality, return, user and match elements\n\t%s\n", inputbuffer);
 					}
 				} else if (headertype == CONFIG_OUTPUT) {
 					if (isfilename(inputbuffer)) {
-						if (!open_audit_output
-						    (inputbuffer)) {
-							DebugMsg(
-								"WARNING: Could not open the file specified in the audit configuration: %s - Auditing to STDOUT\n",
-								inputbuffer);
+						if (!open_audit_output(inputbuffer)) {
+							DebugMsg("WARNING: Could not open the file specified in the audit configuration: %s - Auditing to STDOUT\n", inputbuffer);
 						}
 					} else if (isnetwork(inputbuffer)) {
-						if (!open_audit_network
-						    (inputbuffer)) {
-							DebugMsg(
-								"WARNING: Could not open the network specified in the audit configuration: %s - Auditing to STDOUT\n",
-								inputbuffer);
+						if (!open_audit_network(inputbuffer)) {
+							DebugMsg("WARNING: Could not open the network specified in the audit configuration: %s - Auditing to STDOUT\n", inputbuffer);
 						}
-					} else if(issyslog(inputbuffer)) {
-						SyslogDestination=get_syslog_dest(inputbuffer);
-						if(!SyslogDestination) {
-							SyslogDestination=13;
-							DebugMsg("WARNING: Could not establish the correct syslog destination from the audit configuration: %s - Sending to user.notice\n",inputbuffer);
+					} else if (issyslog(inputbuffer)) {
+						SyslogDestination = get_syslog_dest(inputbuffer);
+						if (!SyslogDestination) {
+							SyslogDestination = 13;
+							DebugMsg("WARNING: Could not establish the correct syslog destination from the audit configuration: %s - Sending to user.notice\n", inputbuffer);
+						}
+					} else if (regmatchi(inputbuffer, "^restart=")) {
+						if (regmatchi(inputbuffer, "=1")) {
+							restart_allow = 1;
 						}
 					}
-
 				} else if (headertype == CONFIG_HOSTID) {
-					gethostident(inputbuffer, hostid,
-						     MAX_HOSTID);
+					gethostident(inputbuffer, hostid, MAX_HOSTID);
 				} else if (headertype == CONFIG_REMOTE) {
 					// Grab the remote control stuff here
-					if (regmatchi
-					    (inputbuffer, "^allow=")) {
-						if (regmatchi
-						    (inputbuffer, "=1")) {
+					if (regmatchi(inputbuffer, "^allow=")) {
+						if (regmatchi(inputbuffer, "=1")) {
 							remote_allow = 1;
 						}
-					} else
-					    if (regmatchi
-						(inputbuffer,
-						 "^listen_port=")) {
-						remote_webport =
-						    getport(inputbuffer);
-					} else
-					    if (regmatchi
-						(inputbuffer,
-						 "^restrict_ip=")) {
-						if (!getconfstring
-						    (inputbuffer,
-						     remote_restrictip,
-						     sizeof
-						     (remote_restrictip)))
-						{
-							strncpy
-							    (remote_restrictip,
-							     "",
-							     sizeof
-							     (remote_restrictip));
+					} else if (regmatchi(inputbuffer, "^listen_port=")) {
+						remote_webport = getport(inputbuffer);
+					} else if (regmatchi(inputbuffer, "^restrict_ip=")) {
+						if (!getconfstring(inputbuffer, remote_restrictip, sizeof(remote_restrictip))) {
+							strncpy(remote_restrictip, "", sizeof(remote_restrictip));
 						}
-					} else
-					    if (regmatchi
-						(inputbuffer,
-						 "^accesskey=")) {
-						if (!getconfstring
-						    (inputbuffer,
-						     remote_password,
-						     sizeof
-						     (remote_password))) {
-							strncpy
-							    (remote_password,
-							     "",
-							     sizeof
-							     (remote_password));
+					} else if (regmatchi(inputbuffer, "^accesskey=")) {
+						if (!getconfstring(inputbuffer, remote_password, sizeof(remote_password))) {
+							strncpy(remote_password, "", sizeof(remote_password));
 						}
 					}
 				} else {
@@ -469,7 +523,7 @@ int read_config_file(int shortread)
 int iscomment(char *line)
 {
 	// Verify that there is something to check.
-	if (line == (char *)NULL) return(1);
+	if (line == (char *) NULL) return(1);
 
 	// And that there is some data within.
 	if (strlen(line) == 0 || strlen(line) > MAX_AUDIT_CONFIG_LINE) return(1);
@@ -520,7 +574,7 @@ int isnetwork(char *string)
 
 int issyslog(char *string)
 {
-	if(regmatch(string, "^syslog=")) {
+	if (regmatch(string, "^syslog=")) {
 		return(1);
 	}
 	return(0);
@@ -530,12 +584,11 @@ int open_audit_output(char *string)
 {
 	string += strlen("file=");
 	if (strlen(string)) {
-		if (regmatchi(string, "stdout")) {
+		if (regmatchi(string,"stdout")) {
 			AuditDestination |= AUDIT_TO_STDOUT;
 			return(1);
 		}
 		AuditFile = fopen(string, "a");
-		DebugMsg("Openning log file: %s\n", string);
 		if (AuditFile != (FILE *) NULL) {
 			AuditDestination |= AUDIT_TO_FILE;
 			return(1);
@@ -569,6 +622,7 @@ int open_audit_network(char *string)
 	int port = 6161, protocol=SOCKETTYPE_UDP;	// Default port, protocol
 	char *pos, *pos2;
 	struct hostent *hp;
+	pthread_t thread; 
 
 	// Initialise desthost.
 	strncpy(desthost, "localhost", MAX_HOSTID);
@@ -636,8 +690,9 @@ int open_audit_network(char *string)
 			hcurrent->cacheflag = 0;
 			hhead=hcurrent;
 			htail=hcurrent;
+
 			AuditDestination |= AUDIT_TO_NETWORK;
-			if(port == 514) {
+			if (port == 514) {
 				AuditDestination |= AUDIT_TO_SYSLOG;
 			}
 			return(1);
@@ -746,15 +801,15 @@ char *criticalitystring(int criticality, char *string)
 {
 	switch (criticality) {
 		case CRITICALITY_CRITICAL:
-			return(strncpy(string, "critical", MAX_AUDITREC));
+			return(strncpy(string, "critical",MAX_AUDITREC));
 		case CRITICALITY_PRIORITY:
-			return(strncpy(string, "priority", MAX_AUDITREC));
+			return(strncpy(string, "priority",MAX_AUDITREC));
 		case CRITICALITY_WARNING:
-			return(strncpy(string, "warning", MAX_AUDITREC));
+			return(strncpy(string, "warning",MAX_AUDITREC));
 		case CRITICALITY_INFO:
-			return(strncpy(string, "information", MAX_AUDITREC));
+			return(strncpy(string, "information",MAX_AUDITREC));
 		default:
-			return(strncpy(string, "clear", MAX_AUDITREC));
+			return(strncpy(string, "clear",MAX_AUDITREC));
 	}
 }
 
@@ -883,21 +938,16 @@ int splitobjective(char *string, char *event, char *user, char *match, int *excl
 	// Pointer to the last character in the string.
 	endstring = &string[strlen(string)];
 
-	if (startevent == (char *) NULL || startuser == (char *) NULL ||
-	    startmatch == (char *) NULL || startcrit == (char *) NULL
-	    || startreturn == (char *) NULL) {
+	if (startevent == (char *)NULL || startuser == (char *)NULL ||
+	   startmatch == (char *)NULL || startcrit == (char *)NULL || startreturn == (char *)NULL) {
 		// Problem, this line is malformed. We really cannot proceed with this line.
 		DebugMsg("The following line does not contain the criticality, event, return, user and match elements: [%s]", string);
 		return(-1);
 	}
+
 	// Start with the event
 	stringpointer = startcrit + strlen("criticality=");
-	while (*stringpointer
-	       && (stringpointer != startcrit
-		   && stringpointer != startreturn
-		   && stringpointer != startuser
-		   && stringpointer != startmatch
-		   && stringpointer != endstring)) {
+	while (*stringpointer && (stringpointer != startcrit && stringpointer != startreturn && stringpointer != startuser && stringpointer != startmatch && stringpointer != endstring)) {
 		*critp = *stringpointer;
 		critp++;
 		stringpointer++;
@@ -910,12 +960,7 @@ int splitobjective(char *string, char *event, char *user, char *match, int *excl
 
 	// Start with the event
 	stringpointer = startevent + strlen("event=");
-	while (*stringpointer
-	       && (stringpointer != startcrit
-		   && stringpointer != startreturn
-		   && stringpointer != startuser
-		   && stringpointer != startmatch
-		   && stringpointer != endstring)) {
+	while (*stringpointer && (stringpointer != startcrit && stringpointer != startreturn && stringpointer != startuser && stringpointer != startmatch && stringpointer != endstring)) {
 		*eventp = *stringpointer;
 		eventp++;
 		stringpointer++;
@@ -940,12 +985,7 @@ int splitobjective(char *string, char *event, char *user, char *match, int *excl
 		stringpointer = startuser + strlen("user=");
 	}
 
-	while (*stringpointer
-	       && (stringpointer != startcrit
-		   && stringpointer != startreturn
-		   && stringpointer != startmatch
-		   && stringpointer != startevent
-		   && stringpointer != endstring)) {
+	while (*stringpointer && (stringpointer != startcrit && stringpointer != startreturn && stringpointer != startmatch && stringpointer != startevent && stringpointer != endstring)) {
 		*userp = *stringpointer;
 		stringpointer++;
 		userp++;
@@ -960,12 +1000,7 @@ int splitobjective(char *string, char *event, char *user, char *match, int *excl
 		stringpointer = startmatch + strlen("match=");
 	}
 
-	while (*stringpointer
-	       && (stringpointer != startcrit
-		   && stringpointer != startreturn
-		   && stringpointer != startevent
-		   && stringpointer != startuser
-		   && stringpointer != endstring)) {
+	while (*stringpointer && (stringpointer != startcrit && stringpointer != startreturn && stringpointer != startevent && stringpointer != startuser && stringpointer != endstring)) {
 		*matchp = *stringpointer;
 		stringpointer++;
 		matchp++;
@@ -1044,9 +1079,7 @@ int IsListEmpty(void)
 		return 0;
 }
 
-Node *AddToList(char *eventname, char *username, char *path,
-		int criticality, char *options, int excludeflag, int excludematchflag,
-		int returncode)
+Node *AddToList(char *eventname, char *username, char *path, int criticality, char *options, int excludeflag, int excludematchflag, int returncode)
 {
 	Node *newNode = NULL;
 
@@ -1187,6 +1220,12 @@ void DestroyList(void)
 		free(MCCurrent);
 		MCCurrent=MCHead;
 	}
+	EvtCurrent=EvtHead;
+	while(EvtCurrent) {
+		EvtHead=EvtCurrent->next;
+		free(EvtCurrent);
+		EvtCurrent=EvtHead;
+	}
 }
 
 // Pull out the fully qualified domain name, if possible.
@@ -1205,16 +1244,12 @@ char *getfqdn(char *FQDN)
 	hp = gethostbyname(hname);
 	if (hp) {
 		while (hp->h_aliases && *hp->h_aliases) {
-			if (strlen(*(hp->h_aliases)) > strlen(hname)
-			    && !strncmp(hname, *(hp->h_aliases),
-					strlen(hname))) {
-				strncpy(FQDN, *(hp->h_aliases),
-					MAX_HOSTID);
+			if (strlen(*(hp->h_aliases)) > strlen(hname) && !strncmp(hname, *(hp->h_aliases), strlen(hname))) {
+				strncpy(FQDN, *(hp->h_aliases), MAX_HOSTID);
 			}
 			hp->h_aliases++;
 		}
-		if (strlen(hp->h_name) > strlen(hname)
-		    && !strncmp(hname, hp->h_name, strlen(hname))) {
+		if (strlen(hp->h_name) > strlen(hname) && !strncmp(hname, hp->h_name, strlen(hname))) {
 			strncpy(FQDN, hp->h_name, MAX_HOSTID);
 		}
 	}
@@ -1233,17 +1268,30 @@ int SendToSocket(char *message)
 			// The socket is broken, flag an error and let last_error deal with it
 			myhostcurrent->last_error=time(&myhostcurrent->last_error);
 		}
+		if (myhostcurrent->last_error) {
+			if (fix_last_error(myhostcurrent) != 0) cachemsg(myhostcurrent,message);
+		}
 		if (myhostcurrent->AuditSocket && !myhostcurrent->last_error) {
 			// Send the packet to each of the destinations
 			// defined in the linked list.
-				//DebugMsg("About to send message\n");
+			if (myhostcurrent->protocol == SOCKETTYPE_UDP || IsSocketValid(myhostcurrent->AuditSocket)) {
 				bytessent = sendto(myhostcurrent->AuditSocket, message, strlen(message), 0, (const struct sockaddr *)&myhostcurrent->AuditSocketName, myhostcurrent->dest_addr_size);
+			} else {
+				// Something is wrong with the socket, but we don't know what
+				// so close it and flag an error
+				DebugMsg("Socket error connecting to %s. Retrying in %d seconds\n", myhostcurrent->desthost, WAITTIME);
+				close(myhostcurrent->AuditSocket);
+				myhostcurrent->last_error=time(&myhostcurrent->last_error);
+				bytessent=-1;
+			}
 
-			//DebugMsg("message sent\n");
-			//DebugMsg("Bytes sent to %s: %d\n", myhostcurrent->desthost, bytessent);
+			// DebugMsg("Bytes sent to %s: %d\n", myhostcurrent->desthost, bytessent);
 			if (bytessent < 0) {
 				DebugMsg("Error sending to server %s. Retrying in %d seconds\n", myhostcurrent->desthost, WAITTIME);
+				// Create another cache entry
 				myhostcurrent->last_error=time(&myhostcurrent->last_error);
+				cachemsg(myhostcurrent, message);
+				caught_pipe = 0;
 			}
 		}
 		myhostcurrent=myhostcurrent->next;
@@ -1251,6 +1299,59 @@ int SendToSocket(char *message)
 	return(0);
 }
 
+int cachemsg (HostNode * hn, char * msg) {
+	return(0);
+}
+
+void purge_cache() {
+	int bsent;
+	struct timespec timeout;
+	time_t now;
+	MsgCache *mymsg;
+
+	//DebugMsg("Starting CACHE purge\n");
+	pthread_mutex_lock(&data_mutex);
+	while (!caught_kill) {
+		// Waiting for head to be valid
+		time(&now);
+		// Run this thread every second, or when signaled.
+		timeout.tv_sec = now + WAITTIME;
+		timeout.tv_nsec = 0;
+		// Ok, check for more data
+
+		pthread_cond_timedwait(&list_data_available,&data_mutex,&timeout);
+		mymsg = MCHead;
+		if (hhead->last_error) fix_last_error(hhead);
+		while (mymsg && !caught_kill && !hhead->last_error) {
+			//if (!hhead->last_error) break;
+			if (hhead->AuditSocket && IsSocketValid(hhead->AuditSocket)) {
+				//DebugMsg("About to send CACHE message\n");
+				bsent = send(hhead->AuditSocket, mymsg->msg, mymsg->msglen, MSG_NONBLOCK);
+			} else {
+			       bsent = -1;
+			}
+			if (bsent < 0) {
+				DebugMsg("Error sending cache to server. Going back to sleep\n", MCCount);
+				// set the last error and then go back to the start
+				close(hhead->AuditSocket);
+				hhead->last_error=time(&hhead->last_error);
+				break;
+			}
+			pthread_mutex_lock(&list_mutex);
+			MCHead = mymsg->next;
+			free(mymsg->msg);
+			free(mymsg);
+			MCCount--;
+			mymsg = MCHead;
+			if (!mymsg) {
+				MCTail=NULL;
+			}
+			pthread_mutex_unlock(&list_mutex);
+		}
+	}
+	pthread_mutex_unlock(&data_mutex);
+}
+
 int fix_last_error(HostNode *myhostcurrent) {
 	time_t now;
 	time(&now);
@@ -1259,7 +1360,7 @@ int fix_last_error(HostNode *myhostcurrent) {
 		if(myhostcurrent->AuditSocket) close(myhostcurrent->AuditSocket);
 		myhostcurrent->AuditSocket=0;
 		myhostcurrent->AuditSocket = socket(AF_INET,SOCK_DGRAM,0);
-		if(myhostcurrent->AuditSocket < 0) {
+		if (myhostcurrent->AuditSocket < 0) {
 			// failed to grab a socket, go back to sleep
 			DebugMsg("Cannot open network socket - continuing to wait\n");
 			myhostcurrent->last_error=time(&myhostcurrent->last_error);
@@ -1268,7 +1369,7 @@ int fix_last_error(HostNode *myhostcurrent) {
 		}
 		myhostcurrent->last_error = 0;
 	} else {
-		//still waiting
+		// still waiting
 		return(1);
 	}
 	return(0);
@@ -1306,9 +1407,6 @@ int sendevent(char *string, int criticality)
 		EventCounter = 1;
 	}
 
-	if (AuditDestination & AUDIT_TO_SYSLOG) {
-		strncpy(syslogout,string,MAX_AUDITREC);
-	}
 	size=strlen(string);
 
 	// Work through our string, and convert it into tab-delimited fields.
@@ -1387,10 +1485,14 @@ int sendevent(char *string, int criticality)
 	}
 	// Ok, string is now broken up with null terminators. Go through and collect our items of interest.
 	
+	if (AuditDestination & AUDIT_TO_SYSLOG) {
+		//strncpy(syslogout,string,MAX_AUDITREC);
+		snprintf(syslogout, MAX_AUDITREC, "%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", EventCounter, startdate,startevent,startcommand,startuser,startrealuser,startpid,startppid,startreturn,starttrail);
+	}
 
 	// NEW audit record format, including host and AIXAudit identifier string:
-	snprintf(stringout, MAX_AUDITREC, "%s\tAIXAudit\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s", hostid,
-		 criticality, EventCounter, startdate,startevent,startcommand,startuser,startrealuser,startpid,startppid,startreturn,starttrail);
+	snprintf(stringout, MAX_AUDITREC, "%s\tAIXAudit\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", hostid,
+	criticality, EventCounter, startdate,startevent,startcommand,startuser,startrealuser,startpid,startppid,startreturn,starttrail);
 
 	EventCounter++;
 
@@ -1401,7 +1503,6 @@ int sendevent(char *string, int criticality)
 
 	if (AuditDestination & AUDIT_TO_FILE) {
 		// Write to file
-		//DebugMsg("writing to file\n");
 		if (AuditFile != (FILE *) NULL) {
 			fprintf(AuditFile, "%s\n", stringout);
 			fflush(AuditFile);
@@ -1409,7 +1510,6 @@ int sendevent(char *string, int criticality)
 	}
 
 	if (AuditDestination & AUDIT_TO_NETWORK) {
-		//DebugMsg("writing to network\n");
 		if (AuditDestination & AUDIT_TO_SYSLOG) {
 			// Do Syslog stuff here.
 			time_t currenttime;
@@ -1439,29 +1539,16 @@ Node *CheckObjective(char *username, char *searchterm, char *eventname, char *op
 
 	while (IsValidItem()) {
 		currentnode = GetCurrentItem();
-		if ((!strcmp(eventname, currentnode->event_name)
-		     || !strcmp(currentnode->event_name, "AUDIT_ALL"))
-		    && (returncode == currentnode->returncode
-			|| currentnode->returncode == RETURNCODE_ANY)) {
+
+		if ((!strcmp(eventname, currentnode->event_name) || !strcmp(currentnode->event_name, "AUDIT_ALL")) && (returncode == currentnode->returncode || currentnode->returncode == RETURNCODE_ANY)) {
 			// Are we including users, or excluding.
+			// if (excludeflag && !regmatchC || !excludeflag && regmatchC) {
 			if (currentnode->excludeflag ^ regmatchC(username, currentnode->usernameRE)) {
+				// if (excludematch && !regmatchC || !excludematch && regmatchC) {
 				if (currentnode->excludematchflag ^ regmatchC(searchterm, currentnode->pathRE)) {
-					if (regmatchC
-					    (options,
-					     currentnode->optionsRE)) {
-						NextItemInList();
-						return(currentnode);
-					}
-				}
-			} else {
-				if (regmatchC
-				    (username, currentnode->usernameRE)
-				    && regmatchC(searchterm,
-						 currentnode->pathRE)) {
-					if (regmatchC
-					    (options,
-					     currentnode->optionsRE)) {
+					if (regmatchC(options, currentnode->optionsRE)) {
 						NextItemInList();
+	DebugMsg("MATCH");
 						return(currentnode);
 					}
 				}
@@ -1626,30 +1713,30 @@ void syslogdate(char *sdate, struct tm *cdate)
 	}
 
 	if (cdate->tm_mday < 10) {
-		snprintf(Date,3," %d%c",cdate->tm_mday,0);
+		snprintf(Date, 3, " %d%c", cdate->tm_mday,0);
 	} else {
-		snprintf(Date,3,"%d%c",cdate->tm_mday,0);
+		snprintf(Date, 3, "%d%c", cdate->tm_mday,0);
 	}
 
-	if(cdate->tm_hour<10) {
-		snprintf(Hour,3,"0%d%c",cdate->tm_hour,0);
+	if (cdate->tm_hour < 10) {
+		snprintf(Hour, 3, "0%d%c", cdate->tm_hour,0);
 	} else {
-		snprintf(Hour,3,"%d%c",cdate->tm_hour,0);
+		snprintf(Hour, 3, "%d%c", cdate->tm_hour,0);
 	}
 
-	if(cdate->tm_min<10) {
-		snprintf(Min,3,"0%d%c",cdate->tm_min,0);
+	if (cdate->tm_min < 10) {
+		snprintf(Min, 3, "0%d%c", cdate->tm_min,0);
 	} else {
-		snprintf(Min,3,"%d%c",cdate->tm_min,0);
+		snprintf(Min, 3, "%d%c", cdate->tm_min,0);
 	}
 
-	if(cdate->tm_sec<10) {
-		snprintf(Sec,3,"0%d%c",cdate->tm_sec,0);
+	if (cdate->tm_sec < 10) {
+		snprintf(Sec, 3, "0%d%c", cdate->tm_sec,0);
 	} else {
-		snprintf(Sec,3,"%d%c",cdate->tm_sec,0);
+		snprintf(Sec, 3, "%d%c", cdate->tm_sec,0);
 	}
 
-	snprintf(sdate,16,"%s %s %s:%s:%s%c",Month,Date,Hour,Min,Sec,0);
+	snprintf(sdate, 16, "%s %s %s:%s:%s%c", Month, Date, Hour, Min, Sec,0);
 }
 
 int IsSocketValid(int sock) {
@@ -1674,6 +1761,7 @@ int IsSocketValid(int sock) {
 
 void DebugMsg(const char *pszFormat, ...)
 {
+#ifdef DEBUG
 	char buf[8192] = "";
 	FILE *fp;
 
@@ -1682,13 +1770,42 @@ void DebugMsg(const char *pszFormat, ...)
 	vsnprintf(&buf[strlen(buf)], 8192 - strlen(buf) - 1, pszFormat, arglist);
 	va_end(arglist);
 
-	fp=fopen("/home/u0000190/SNARE-OUT", "a");
+	fp=fopen("/tmp/SNARE-OUT", "a");
 	if (fp) {
 		fputs(buf,fp);
+		fputs("\n",fp);
 		fclose(fp);
 	} else {
 		printf("\nERROR: There was an error opening the log file\n");
 	}
 	//printf("%s\n", buf);
 	//fflush(stdout);
+#endif //DEBUG
+}
+
+void start_web () {
+	int ret=0;
+	time_t lasttime=(time_t)0;
+	// Child process - Run our web server
+	DebugMsg("web init %d", pthread_self());
+	while(InitWebServer(remote_webport,remote_restrictip,remote_password) == -1) {
+		// Problem binding. Sleep for a moment.
+		sleep(5);
+	}
+	DebugMsg("web start");
+
+	while (!caught_kill && !caught_pipe) {
+		ret = ListenThread(&lasttime);
+
+		//if (!ret) break;
+	}
+	CloseWebServer();
+	//while((ret = StartThread()) == 1) {
+		// Problem binding. Keep trying.
+		//sleep(5);
+	//}
+#ifdef DEBUG
+	DebugMsg("Web server is RETURNING");
+#endif //DEBUG
+	pthread_exit(NULL);
 }
