@@ -1,10 +1,12 @@
+#ifndef _SNARECORE_H
+#define _SNARECORE_H
 /**********************************************************
  *
  * Snare for AIX header file
  *
  * Author: InterSect Alliance Pty Ltd
  *
- * Copyright 2001-2004 InterSect Alliance Pty Ltd
+ * Copyright 2001-2007 InterSect Alliance Pty Ltd
  *
  * Last Modified: 7/12/2004
  *
@@ -19,8 +21,9 @@
  **********************************************************/
 
 // Linked List
-struct _node {
-	// int event_number;    // The event number this is supposed to match
+struct _node
+{
+	// int event_number;	// The event number this is supposed to match
 	char event_name[MAX_EVENTNAME];	// The event number this is supposed to match
 	int criticality;	// How critical is this particular node
 	int returncode;		// return code required.
@@ -46,6 +49,24 @@ struct _msgcache {
 
 typedef struct _msgcache MsgCache;
 
+struct _webcache {
+	char hostid[MAX_HOSTID];
+	//char string[MAX_AUDITREC];
+	char CurrentDate[16];
+	char username[MAX_USERNAME];
+	char searchterm[MAX_PATH];
+	char eventname[MAX_EVENTNAME];
+	char options[MAX_OPTIONS];
+	int returncode;
+	int criticality;
+	int snareseq;
+	int seenflag;
+	struct _webcache *next;
+	struct _webcache *prev;
+};
+
+typedef struct _webcache WebCache;
+
 struct _hostnode
 {
 	int AuditSocket;
@@ -61,36 +82,24 @@ struct _hostnode
 
 typedef struct _hostnode HostNode;
 
-int turn_event_on(int);
-int turn_event_off(int);
-int setclass(unsigned int);
-int read_config_file(int);
-char *gethostident(char *string, char *host, int length);
-char *getconfstring(char *string, char *file, int length);
-int getport(char *string);
+void	pipe_signal();		/* Routine to handle SIGPIPEs */
+void	kill_signal();		/* Routine to handle SIGTERMs */
+void	usr1_signal();
 
+int		turn_event_on(int);
+int		turn_event_off(int);
+int		setclass(unsigned int);
+int		read_config_file(int);
+char *	gethostident(char *string, char *host,int length);
+char *	getconfstring(char *string, char *file,int length);
+int		getport(char *string);
+
+int		setsignals(sigset_t *signalset);
 int flush_audit_events(void);
 
-void trim(char *string);
-void trimallwhitespace(char *string);
-int IsListEmpty(void);
-
-Node *AddToList(char *eventname, char *username, char *path,
-		int criticality, char *options, int excludeflag,
-		int excludematchflag, int returncode);
-char *getfqdn(char *FQDN);
-
-Node *CheckObjective(char *username, char *searchterm, char *eventname,
-		     char *options, int returncode);
-char *FormatDelimiters(char *eventbuffer);
-int GetAIXDetails(char *logbuffer, char *username, char *searchterm,
-		  char *eventname, char *options);
-int GetElement(char *source, int count, char *dest, int buffersize);
-Node *GetCurrentItem(void);
-
-char *FindToken(char *,char *);
-
-void DestroyList(void);
+void	trim(char *string);
+void	trimallwhitespace(char *string);
+int		IsListEmpty(void);
 
 int audit_off(void);
 int iscomment(char *);
@@ -107,12 +116,29 @@ int issyslog(char *);
 int get_syslog_dest(char *);
 
 
+
+Node *	AddToList(char *eventname, char *username, char *path, int criticality, char *options, int excludeflag, int excludematchflag, int returncode);
+char *	getfqdn(char * FQDN);
+
+Node *	CheckObjective(char *username,char *searchterm, char *eventname, char *options, int returncode);
+char *	FormatDelimiters(char *eventbuffer);
+int		GetAIXDetails(char *logbuffer, char *username, char *searchterm, char *eventname, char *options);
+int GetElement(char *source, int count, char *dest, int buffersize);
+Node *GetCurrentItem(void);
+
+char *FindToken(char *,char *);
+
+void DestroyList(void);
+int IsSocketValid(int socket);
+int cachemsg(HostNode *hn, char * msg);
+void purge_cache();
+int fix_last_error(HostNode *hn);
+void start_web ();
+
 int open_audit_network(char *);
 int open_audit_output(char *);
 
 extern int StartThread(void);
 
 void DebugMsg(const char *,...);
-int IsSocketValid(int socket);
-void * purge_cache();
-int fix_last_error(HostNode *hn);
+#endif //_SNARECORE_H
